/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.apache.woden.wsdl20.xml;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.woden.internal.wsdl20.DescriptionImpl;

/**
 * Unit tests for DocumentationImpl class.
 * 
 * @author John Kaputin (jkaputin@apache.org)
 *
 */
public class DocumentationElementTest extends TestCase {

    public static Test suite()
    {
       return new TestSuite(DocumentationElementTest.class);
    }
       

    /**
     * Test method for {@link org.apache.woden.internal.wsdl20.DocumentationImpl#setContent(java.lang.Object)}.
     * Test method for {@link org.apache.woden.internal.wsdl20.DocumentationImpl#setContent(java.lang.Object)}.
     */
    public void testSetGetContent() {
        DescriptionElement descElem = new DescriptionImpl();
        DocumentationElement docElem = descElem.addDocumentationElement();
        
        //check that the doc element is empty when first created
        assertNull(docElem.getContent());
        
        //test the setter and getter methods for doc elem content
        docElem.setContent("Doc content");
        assertEquals("Doc content", docElem.getContent());
    }

    /**
     * Test method for {@link org.apache.woden.internal.wsdl20.DocumentationImpl#setParentElement(org.apache.woden.wsdl20.xml.WSDLElement)}.
     * Test method for {@link org.apache.woden.internal.wsdl20.DocumentationImpl#getParentElement()}.
     */
    public void testSetGetParentElement() {
        DescriptionElement descElem1 = new DescriptionImpl();
        DocumentationElement docElem = descElem1.addDocumentationElement();
        
        //check the initial parent
        assertEquals(descElem1, docElem.getParentElement());
        
        DescriptionElement descElem2 = new DescriptionImpl();
        docElem.setParentElement(descElem2);
        
        //test that the parent has been changed by the set method
        assertEquals(descElem2, docElem.getParentElement());
    }

}
